package control

import (
	"context"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/engine"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control/server/ctrlmessage"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (s *Server) FlushCache(ctx context.Context, req *control.FlushCacheRequest) (*control.FlushCacheResponse, error) {
	err := s.isValidRequest(req)
	if err != nil {
		return nil, status.Error(codes.PermissionDenied, err.Error())
	}

	for _, shardID := range s.getShardIDList(req.GetBody().GetShard_ID()) {
		var prm engine.FlushWriteCachePrm
		prm.SetShardID(shardID)
		prm.SetSeal(req.GetBody().GetSeal())

		_, err = s.s.FlushWriteCache(ctx, prm)
		if err != nil {
			return nil, status.Error(codes.Internal, err.Error())
		}
	}

	resp := &control.FlushCacheResponse{Body: &control.FlushCacheResponse_Body{}}

	err = ctrlmessage.Sign(s.key, resp)
	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}
	return resp, nil
}
